/*
Rapid Doji EA 06

This MQL5 Expert Advisor uses a simple strategy on a daily chart.

For step-by-step video tutorials (with transcripts) on how this EA
was built, go to the link below.
*/
#property copyright		"Copyright 2020, Anthony J. Garot"
#property link			"https://www.garot.com/trading/"
#property version		"1.00"
#property description	""
#property description	"Rapid Doji EA 06"
#property description	""
#property description	"Entry: Two pending orders around each Daily Doji bar."
#property description	"Risk adjustment: Fixed trailing SL."
#property strict

#include <Trade\Trade.mqh>
#include <Trade\AccountInfo.mqh>

#define NUM_RATES 3
#define	KEY_Q 81							// Key code for letter 'q'

//--- globals
CTrade			g_trade;					// trade object
double			g_pointSize = 0.;			// initialize to ludicrous value
// handles
int				g_handleAtr = 0;
// arrays
MqlRates		g_rates[];
double			g_atr[];

//--- input parameters
input double	SL_ATR_FACTOR = 1.75;			// ATR Factor to determine distance to place SL
input int		AR_FIXED_TRAILING_PTS = 2700;	// Max points to keep SL from prices while trailing
input int		InpAtrPeriod = 14;				// ATR Period

//--- constants
const long		MAGIC_NUMBER = 5678900;
const double	TRADE_VOLUME = 0.10;			// For strategy development, fixed lot size of 0.10 is good
const int		CHART_ID = 0;					// Current chart
const int		SUB_WINDOW = 0;					// Main Window

int OnInit(void)
{
	g_trade.SetExpertMagicNumber(MAGIC_NUMBER);

	g_pointSize = SymbolInfoDouble(_Symbol, SYMBOL_POINT);
	PrintFormat("g_pointSize is [%.5f]", g_pointSize);

	//--- ATR
	g_handleAtr=iATR(	_Symbol,		// symbol
						_Period,		// period
						InpAtrPeriod );	// atr_period
	if ( g_handleAtr == INVALID_HANDLE )
	{
		PrintFormat("%s(): Error creating iATR indicator handle for [%s] [%d]", _Symbol, _Period );
		return INIT_FAILED;
	}

	ArrayResize(g_rates, NUM_RATES);
	ArraySetAsSeries(g_rates, true);
	ArrayResize(g_atr, NUM_RATES);
	ArraySetAsSeries(g_atr, true);

	//--- ok
	Print("INIT_SUCCEEDED for EA!");
	return(INIT_SUCCEEDED);
}

// handles chart events such as keystrokes and mouse movements
void OnChartEvent(	const int id,			// Event ID
					const long& lparam,		// Parameter of type long event
					const double& dparam,	// Parameter of type double event
					const string& sparam)	// Parameter of type string events, max 159 chars
{
	if( id == CHARTEVENT_KEYDOWN )
	{
		switch(int(lparam))
		{
		case KEY_Q:		// 'q'
			{
				int keystate = TerminalInfoInteger(TERMINAL_KEYSTATE_CONTROL);
				if ( keystate == -127 || keystate == -128 )
				{
					// This is a quick way to exit an Expert
					Print("CTRL-Q hit. Removing Expert");
					ExpertRemove();
				}
			}
			break;
		}
	}
}

// Called right before OnDeinit.
double OnTester()
{
	string			labelText = "";
	int				labelX = -1;												// set to nonsensical value
	int				labelY = 10;												// Initialize from window top, in pixels
	const int		LABEL_HEIGHT = 20;

	// shorthand constants from MT5's stats
	const int		profitTrades = (int) TesterStatistics(STAT_PROFIT_TRADES);	// # count of profitable trades
	const int		lossTrades = (int) TesterStatistics(STAT_LOSS_TRADES);		// # count of losing trades
	const double	totalProfit = TesterStatistics(STAT_GROSS_PROFIT);			// $ amount of profit
	const double	totalLoss = TesterStatistics(STAT_GROSS_LOSS);				// $ amount of loss

	// This captures a final on-going trade when the strategy tester ends.
	if (HistorySelect(TimeCurrent(), INT_MAX) && HistoryDealsTotal())
	{
		const ulong Ticket = HistoryDealGetTicket(HistoryDealsTotal() - 1);
		TradeCompleted(Ticket);
	}

	// Get the width of the Strategy Tester Visualization window
	const int width = (int) ChartGetInteger(CHART_ID,CHART_WIDTH_IN_PIXELS);
	labelX = width - 200;													// set once. 200 seems reasonable for now.

	// The first thing I want to see is the profit.
	labelText = StringFormat("Profit [$%.2f]", TesterStatistics(STAT_PROFIT) );
	labelY = labelY + LABEL_HEIGHT;
	DrawLabel(labelX, labelY, labelText);

	// Calculate some values
	const int totTrades = (profitTrades + lossTrades);
	const double winPct = (totTrades!=0)?100. * profitTrades / totTrades:0.;
	const double lossPct = (totTrades!=0)?100. * lossTrades / totTrades:0.;

	labelText = StringFormat("W [%d](%.1f%%) L [%d](%.1f%%)", profitTrades, winPct, lossTrades, lossPct );
	labelY = labelY + LABEL_HEIGHT;
	DrawLabel(labelX, labelY, labelText);

	labelText = StringFormat("Total Trades [%d]", totTrades );
	labelY = labelY + LABEL_HEIGHT;
	DrawLabel(labelX, labelY, labelText);

	// Less than $20 is typically no good
	const double expectancy = GetExpectancy(profitTrades, lossTrades, totalProfit, totalLoss);
	labelText = StringFormat("Expectancy [$%.2f]", expectancy);
	labelY = labelY + LABEL_HEIGHT;
	DrawLabel(labelX, labelY, labelText);

	// I think there is some value in knowing how many wins, because that tells you if any
	// of your rules actually give you an edge. You can tweak the ATR factor on a trailing SL
	// after that.
	const double winLossRatio = (lossTrades != 0)?(1.0*profitTrades/lossTrades):0.0;
	labelText = StringFormat("Win/Loss Ratio [%.2f]", winLossRatio );
	labelY = labelY + LABEL_HEIGHT;
	DrawLabel(labelX, labelY, labelText);

	// Measures the ability of a trading system to recover from losses.
	// Larger the better . . . >2 or >3 is usually good.
	const double recoveryFactor = TesterStatistics(STAT_RECOVERY_FACTOR);
	labelText = StringFormat("Recovery Factor [%.2f]", recoveryFactor );
	labelY = labelY + LABEL_HEIGHT;
	DrawLabel(labelX, labelY, labelText);

	// "Custom max criterion"
	// Expectancy is unnecessary as as a "Custom max criterion" because it is already
	// included in the XML optimization results. However, win/loss ratio is not.
	double customMaxCriterion = winLossRatio;
	// Return the "Custom max criterion"
	// This value can be captured by TesterStatistics(STAT_CUSTOM_ONTESTER).
	// It is also the "Result" column in the optimization results tab.
	return customMaxCriterion;
}

// Remember that this runs every tick -- unless the tester is running on "Open prices only"
void OnTick(void)
{
	// Bar level functionality
	if ( IsNewBar(_Symbol, _Period) )
	{
		// Need data or we've got nothing
		if ( ! RefreshData() )
		{
			Print("Couldn't refresh data");
			return;
		}

		// See if a position is already open
		if ( SelectPosition(_Symbol, MAGIC_NUMBER) )
		{
			// Position is open, so adjust risk as necessary
			FixedTrailingStop();
		}
		else
		{
			// No position open, so look for signal
			CheckForOpenSignal();
		}

		// Draw every Doji whether we place a trade or not
		if ( IsDoji(g_rates[1]) )
		{
			DrawArrow(g_rates[1].time, g_rates[1].low - .333 * g_atr[1], ARROW_UP, Yellow, "Doji");
		}
	}
}

void OnTradeTransaction(
	const MqlTradeTransaction&		trans,			// trade transaction structure
	const MqlTradeRequest&			request,		// request structure
	const MqlTradeResult&			result			// result structure
	)
{
	if ( trans.deal > 0 )
	{
		HistoryDealSelect(trans.deal);

		// IN
		// Detect when an order is converted to a position, then delete any other pending order
		if ( (ENUM_DEAL_ENTRY)HistoryDealGetInteger(trans.deal,DEAL_ENTRY) == DEAL_ENTRY_IN )
		{
			const ENUM_DEAL_PROPERTY_INTEGER dealTicket=(ENUM_DEAL_PROPERTY_INTEGER) HistoryDealGetInteger(trans.deal, DEAL_TICKET);
			//const ENUM_DEAL_PROPERTY_INTEGER dealOrder=(ENUM_DEAL_PROPERTY_INTEGER) HistoryDealGetInteger(trans.deal, DEAL_ORDER);
			const ENUM_DEAL_TYPE dealType=(ENUM_DEAL_TYPE) HistoryDealGetInteger(trans.deal, DEAL_TYPE);
			const datetime transactionTime=(datetime)HistoryDealGetInteger(trans.deal,DEAL_TIME);
			Print(StringFormat(	"IN -  Tkt [%d] [%s] Type [%s]",
								dealTicket,
								TimeToString(transactionTime, TIME_DATE|TIME_SECONDS),
								EnumToString(dealType)));

			if ( ! OrderDeletePending(_Symbol, MAGIC_NUMBER) )
			{
				Print("ALERT! Could not delete pending orders");
				ExpertRemove();
			}
		}

		// OUT
		// Detect closed trades (due to SL or TP)
		if ( (ENUM_DEAL_ENTRY)HistoryDealGetInteger(trans.deal,DEAL_ENTRY) == DEAL_ENTRY_OUT )
		{
			TradeCompleted(trans.deal);
		}
	}
}

// OPEN LOGIC
// Returns true if a signal was found (not if position opened).
bool CheckForOpenSignal()
{
	// These are passed and filled in by the signal functions
	double			price = 0.;
	double			sl = 0.;			// initialize to no SL
	double			tp = 0.;			// initialize to no TP

	// BUY STOP
	if ( IsPendingBuySignal(price,sl,tp) )
	{
		//--- additional checking
		if ( Bars(_Symbol,_Period)>100 )
		{
			bool opened = OrderStop(	g_trade,
										_Symbol,
										ORDER_TYPE_BUY_STOP,
										TRADE_VOLUME,
										price,
										sl,
										tp );

			// Draw some lines
			if ( opened )
			{
				DrawLine ( sl, Brown, STYLE_DOT );
				DrawLine ( price, LightBlue, STYLE_DOT );
			}
		}
	}

	// SELL STOP
	if ( IsPendingSellSignal(price,sl,tp) )
	{
		//--- additional checking
		if ( Bars(_Symbol,_Period)>100 )
		{
			bool opened = OrderStop(	g_trade,
										_Symbol,
										ORDER_TYPE_SELL_STOP,
										TRADE_VOLUME,
										price,
										sl,
										tp );

			// Draw some lines
			if ( opened )
			{
				DrawLine ( sl, Brown, STYLE_DOT );
				DrawLine ( price, LightBlue, STYLE_DOT );
			}
		}
	}

	return false;
}

//+------------------------------------------------------------------+
//| Checks STOPS levels                                              |
//+------------------------------------------------------------------+
bool CheckStopsLevel(	const ENUM_ORDER_TYPE order_type,
						const string symbol,
						const double price,
						const double sl)
{
	// shorthand constants make code more readable
	const double pointSize = SymbolInfoDouble(symbol, SYMBOL_POINT);
	const double minStopsLevel = SymbolInfoInteger(symbol, SYMBOL_TRADE_STOPS_LEVEL) * pointSize;
	const double ask = SymbolInfoDouble(symbol, SYMBOL_ASK);
	const double bid = SymbolInfoDouble(symbol, SYMBOL_BID);
	double distancePts = -1.;

	switch(order_type)
	{
		case ORDER_TYPE_BUY:
		{
			if(sl != NULL)
			{
				// SL distancePts to price.
				distancePts = bid - sl;
				if(distancePts <= minStopsLevel)
				{
					Print(StringFormat("%s(): INFO: For BUY, distancePts [%.5f] <= minStopsLevel [%.5f]", __FUNCTION__, distancePts, minStopsLevel));
					return false;
				}
			}
		}
		break;

		case ORDER_TYPE_SELL:
		{
			if(sl != NULL)
			{
				// SL distancePts to price.
				distancePts = (sl - ask);
				if(distancePts <= minStopsLevel)
				{
					Print(StringFormat("%s(): INFO: For SELL, distancePts [%.5f] <= minStopsLevel [%.5f]", __FUNCTION__, distancePts, minStopsLevel));
					return false;
				}
			}
		}
		break;

		case ORDER_TYPE_BUY_STOP:
		{
			if(price < ask)
			{
				Print(StringFormat("%s(): INFO: BUY_STOP price [%.5f] < ask [%.5f]", __FUNCTION__, price, ask));
				return false;
			}

			if(sl != NULL)
			{
				// SL distancePts to price.
				distancePts = (price - sl);
				if(distancePts < minStopsLevel)
				{
					Print(StringFormat("%s(): INFO: For BUY_STOP, distancePts [%.5f] <= minStopsLevel [%.5f]", __FUNCTION__, distancePts, minStopsLevel));
					return false;
				}
			}
		}
		break;

		case ORDER_TYPE_SELL_STOP:
		{
			if(price > bid)
			{
				Print(StringFormat("%s(): INFO: SELL_STOP price [%.5f] > bid [%.5f]", __FUNCTION__, price, bid));
				return false;
			}

			if(sl != NULL)
			{
				// SL distancePts to price.
				distancePts = (sl - price);
				if(distancePts < minStopsLevel)
				{
					Print(StringFormat("%s(): INFO: For SELL_STOP, distancePts [%.5f] <= minStopsLevel [%.5f]", __FUNCTION__, distancePts, minStopsLevel));
					return false;
				}
			}
		}
		break;

		default:
			Print(StringFormat("%s(): Unknown order_type [%s]", __FUNCTION__, EnumToString(order_type)));
			return false;
	}

	return true;
}

const uchar ARROW_UP = 233;
const uchar ARROW_DOWN = 234;
void DrawArrow(const datetime time, double price, const uchar code, const color clr, string comment)
{

	static int s_id = 0;		// to make unique string
	const string name = StringFormat("Arrow_%2d %s", s_id++, comment);

	if (!ObjectCreate( CHART_ID, name, OBJ_ARROW, SUB_WINDOW, time, price ))
	{
		PrintFormat("%s(): Failed to create OBJ_TEXT. Error code: [%d]", __FUNCTION__, GetLastError() );
		return;
	}

	ObjectSetInteger(CHART_ID, name, OBJPROP_COLOR,clr);
	ObjectSetInteger(CHART_ID, name, OBJPROP_ARROWCODE, code);
	ObjectSetInteger(CHART_ID, name, OBJPROP_WIDTH, 3);
}

// Used for box backgrounds and such.
void DrawLabel ( const int x, const int y, const string text )
{
	static int s_id = 0;		// to make unique string
	string name = StringFormat("Label_%2d", s_id++);

	if ( ! ObjectCreate(CHART_ID,name,OBJ_LABEL,SUB_WINDOW,0,0) )
	{
		PrintFormat("%s(): Failed to create OBJ_LABEL. Error code: [%d]", __FUNCTION__, GetLastError() );
		return;
	}

	ObjectSetInteger(CHART_ID, name, OBJPROP_XDISTANCE,		x);
	ObjectSetInteger(CHART_ID, name, OBJPROP_YDISTANCE,		y);
	ObjectSetString (CHART_ID, name, OBJPROP_TEXT,			text);
	ObjectSetInteger(CHART_ID, name, OBJPROP_COLOR,			Plum);
}

void DrawLine( const double price, const color clr = Plum, const ENUM_LINE_STYLE lineStyle = STYLE_SOLID )
{
	static int s_id = 0;			// to make unique string
	const string name = StringFormat("Line_%2d Price [%.5f]", s_id++, price);

	if (!ObjectCreate( CHART_ID, name, OBJ_TREND, SUB_WINDOW, g_rates[2].time, price, g_rates[0].time, price))
	{
		PrintFormat("%s(): Failed to create OBJ_TREND. Error code: [%d]", __FUNCTION__, GetLastError() );
		return;
	}

	ObjectSetInteger(CHART_ID,name,OBJPROP_COLOR,clr);
	ObjectSetInteger(CHART_ID,name,OBJPROP_STYLE,lineStyle);
}

// Trailing stop
// Risk handled at the bar level.
bool FixedTrailingStop(void)
{
	CPositionInfo	g_positionInfo;			// MQL standard library for positions
	double			slDPts = 0.;			// SL distance points
	double			plDPts = 0.;			// Profit/Loss distance points

	// Get the step amount.
	// This sort of assumes that the original SL was set at N*ATR in the first place;
	// otherwise, you will get a big jump the first time, or it won't move at all.
	const double fixedDPts = AR_FIXED_TRAILING_PTS * g_pointSize;

	double price = 0.;							// Current price (ASK or BID)
	const double open = g_positionInfo.PriceOpen();
	const double sl = g_positionInfo.StopLoss();
	const ENUM_POSITION_TYPE positionType = g_positionInfo.PositionType();

	switch(positionType)
	{
		case POSITION_TYPE_BUY:
			price = SymbolInfoDouble(_Symbol,SYMBOL_BID);
			slDPts = price - sl;
			plDPts = price - open;		// Profit Loss is determined from the open
			break;
		case POSITION_TYPE_SELL:
			price = SymbolInfoDouble(_Symbol,SYMBOL_ASK);
			slDPts = sl - price;
			plDPts = open - price;
			break;
		default:
			PrintFormat("Unknown position type [%s]", EnumToString(positionType));
			ExpertRemove();
	}

	// Is the distance from slDPts (price to SL) greater than the trailing ATR distance?
	if ( slDPts > fixedDPts )
	{
		// Never move SL when unprofitable, e.g. lower volatility.
		const double spread = (double) SymbolInfoInteger(_Symbol,SYMBOL_SPREAD) * g_pointSize;
		if ( plDPts < spread )
		{
			// not profitable
			return false;
		}

		double newSL = -1.;		// Start with ridiculous value
		switch(positionType)
		{
			case POSITION_TYPE_BUY:
				newSL = g_rates[1].close - fixedDPts;
				break;
			case POSITION_TYPE_SELL:
				newSL = g_rates[1].close + fixedDPts;
				break;
			default:
				PrintFormat("Unknown position type [%s]", EnumToString(positionType));
				ExpertRemove();
		}

		// Don't request a move if the new SL is essentially the same as the old one.
		if(MathAbs(newSL - sl) < g_pointSize)
		{
			return false;
		}

		// Move SL absolute
		bool modified = PositionModify( g_trade, g_positionInfo, _Symbol, newSL, NULL );
		if ( modified )
		{
			DrawLine(newSL, Red);
		}
		return modified;
	}
	return false;
}

//+------------------------------------------------------------------+
//|																	 |
//+------------------------------------------------------------------+
double FixLotSize(const double lotSize, const string symbol)
{
	// Lot STEP
	double lotStep = SymbolInfoDouble(symbol, SYMBOL_VOLUME_STEP);

	double normalizedLotSize = MathFloor(lotSize / lotStep) * lotStep;

	double minLot = SymbolInfoDouble(symbol, SYMBOL_VOLUME_MIN);
	double maxLot = SymbolInfoDouble(symbol, SYMBOL_VOLUME_MAX);
	if(normalizedLotSize < minLot)
		normalizedLotSize = minLot;
	if(normalizedLotSize > maxLot)
		normalizedLotSize = maxLot;

	return normalizedLotSize;
}

// Expectancy = ( % win * avg win $) + ( % loss * avg loss $)
//    % win       = winning percentage, expressed as decimal, e.g. 0.30 is 30%.
//    avg win $   = dollar value of average winning trade
//    % loss      = losing percentage, expressed as decimal
//    avg loss $  = dollar value of average losing trade (must be less than zero)
//
// Less than $20 is typically no good
double GetExpectancy(const int profitTrades, const int lossTrades, const double totalProfit, const double totalLoss)
{
	const int totalTrades = profitTrades + lossTrades;
	double percentageWin = (totalTrades!=0)?1.0 * profitTrades / totalTrades:0.;
	double percentageLose = (totalTrades!=0)?1.0 * lossTrades / totalTrades:0.;
	double winAvg = 0;
	double loseAvg = 0;

	if ( profitTrades > 0 )
	{
		winAvg = 1.0 * totalProfit / profitTrades;
	}
	if ( lossTrades > 0 )
	{
		loseAvg = 1.0 * totalLoss / lossTrades;
	}

	PrintFormat("EXPECT:HEY # profit trades [%d] # loss trades [%d] # total trades [%d]",
				profitTrades,
				lossTrades,
				totalTrades
				);

	PrintFormat("EXPECT:HEY %% win [%.2f] %% lose [%.2f] Av$ Win [%.2f] Av$ Lose [%.2f]",
				percentageWin,
				percentageLose,
				winAvg,
				loseAvg
				);

	return (percentageWin * winAvg) + (percentageLose * loseAvg);
}

// Body <= 3% of Bar
// Does not consider position of body, e.g. gravestone or dragonfly
bool IsDoji(MqlRates& rate)
{
	const double DOJI_PCT = 0.03;		// Assume 3% or less body size for Doji

	const double sizeBody = MathAbs( rate.open - rate.close );
	const double sizeBar = ( rate.high - rate.low );
	return ( sizeBody <= DOJI_PCT * sizeBar );
}

// Detects when a "new bar" occurs, which is the same as when the previous bar has completed.
bool IsNewBar(const string symbol, const ENUM_TIMEFRAMES period)
{
	bool isNewBar = false;
	static datetime priorBarOpenTime = NULL;

	// New Bar event handler -> per https://www.mql5.com/en/articles/159
	// SERIES_LASTBAR_DATE == Open time of the last bar of the symbol-period
	const datetime currentBarOpenTime = (datetime) SeriesInfoInteger(symbol,period,SERIES_LASTBAR_DATE);

	if( priorBarOpenTime != currentBarOpenTime )
	{
		// Don't want new bar just because EA started
		isNewBar = ( priorBarOpenTime == NULL )?false:true;	// priorBarOpenTime is only NULL once

		// Regardless of new bar, update the held bar time
		priorBarOpenTime = currentBarOpenTime;
	}

	return isNewBar;
}

bool IsPendingBuySignal(double &price,double &sl,double &tp)
{
	// STRATEGY
	if ( ! IsDoji(g_rates[1]) ) return false;

	price=g_rates[1].high;
	sl = g_rates[1].low - SL_ATR_FACTOR * g_atr[1];

	return true;
}

bool IsPendingSellSignal(double &price,double &sl,double &tp)
{
	// STRATEGY
	if ( ! IsDoji(g_rates[1]) ) return false;

	price=g_rates[1].low;
	sl = g_rates[1].high + SL_ATR_FACTOR * g_atr[1];

	return true;

}

bool OrderDeletePending(const string symbol, const long magicNumber)
{
	COrderInfo order;

	int numOrders = OrdersTotal();
	for(int i=numOrders-1;i>=0;i--)
	{
		if(order.SelectByIndex(i))
		{
			// It appears that ORDER_MAGIC doesn't exist until the order is closed. ???
			if(order.Symbol()==symbol && order.Magic()==magicNumber)
			{
				switch ( order.OrderType() )
				{
					// For any pending order type
					case ORDER_TYPE_BUY_STOP:
					case ORDER_TYPE_SELL_STOP:
					case ORDER_TYPE_BUY_LIMIT:
					case ORDER_TYPE_SELL_LIMIT:
						PrintFormat(	"	%s(): INFO:About to delete PND order, ticket [%d] type [%s]",
											__FUNCTION__,
											order.Ticket(),
											EnumToString(order.OrderType()) );

						if ( ! g_trade.OrderDelete(order.Ticket()) )
						{
							return false;
						}
						break;
					default:
						PrintFormat("Unknown order type [%s]", EnumToString(order.OrderType()));
						//ExpertRemove();
				}
			}
		}
	}

	return true;
}

bool OrderStop(		CTrade& trade,
					const string symbol,
					const ENUM_ORDER_TYPE order_type,
					double volume,
					const double price,
					const double sl = 0.,
					const double tp = 0.	)
{

	//--- additional checking
	if(! TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))
	{
		PrintFormat("%s(%s): Error [TERMINAL_TRADE_ALLOWED]", __FUNCTION__, symbol );
		return false;
	}

	// ensure volume is legit, even though it's a constant.
	// This is a good idea anyway; but I put it in this code specifically to pass the MQL5.com validator.
	volume = FixLotSize(volume, symbol);

	// Ensure we have enough money for the trade
	CAccountInfo account;
	const ENUM_ORDER_TYPE direction = (order_type == ORDER_TYPE_BUY_STOP) ? ORDER_TYPE_BUY : ORDER_TYPE_SELL;
	const double freeMargin = account.FreeMarginCheck(symbol, direction, volume, price);
	if(freeMargin < 0.00)
	{
		Print("Error! Not enough money");
		return false;
	}

	// Ensure TP and SL meet minimum server levels.
	if(! CheckStopsLevel(order_type, symbol, price, sl))
	{
		return false;
	}

	// enum to string
	if ( order_type == ORDER_TYPE_BUY_STOP )
	{
		if(!trade.BuyStop(volume,price,symbol,sl,tp, ORDER_TIME_GTC, NULL, NULL ))
		{
			PrintFormat("%s(%s) Error: Retcode [%d] : '%s'", __FUNCTION__,symbol, trade.ResultRetcode(), trade.ResultComment() );
			return false;
		}
	}
	else if ( order_type == ORDER_TYPE_SELL_STOP )
	{
		if(!trade.SellStop(volume,price,symbol,sl,tp, ORDER_TIME_GTC, NULL, NULL ))
		{
			PrintFormat("%s(%s) Error: Retcode [%d] : '%s'", __FUNCTION__,symbol, trade.ResultRetcode(), trade.ResultComment() );
			return false;
		}
	}
	else { PrintFormat("%s(): Print! Unknown order type [%s]", __FUNCTION__, EnumToString(order_type) ); }

	return true;
}

bool PositionModify(	CTrade& trade,
						CPositionInfo& position,
						const string symbol,
						const double sl,
						double tp = NULL )
{

	//--- additional checking
	if(! TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))
	{
		PrintFormat("%s(%s): Error [TERMINAL_TRADE_ALLOWED]", __FUNCTION__, symbol );
		return false;
	}

	if (tp == NULL) { tp = position.TakeProfit(); }
	double price = position.PriceOpen();

	// Ensure TP and SL meet minimum server levels.
	const ENUM_POSITION_TYPE positionType = position.PositionType();
	const ENUM_ORDER_TYPE orderType = (positionType == POSITION_TYPE_BUY) ? ORDER_TYPE_BUY : ORDER_TYPE_SELL;
	if(! CheckStopsLevel(orderType, symbol, price, sl))
	{
		return false;
	}

	// Do a broker modify
	// https://www.mql5.com/en/docs/standardlibrary/tradeclasses/ctrade/ctradepositionmodify
	bool modified = trade.PositionModify(symbol,sl,tp);
	if ( !modified )
	{
		PrintFormat(	"%s(%s) Error: Retcode [%d] : '%s'", __FUNCTION__, symbol, trade.ResultRetcode(), trade.ResultComment() );
		return false;
	}

	return true;
}

bool RefreshData(void)
{
	//--- go trading only for first ticks of new bar
	if(CopyRates(_Symbol,_Period,0,NUM_RATES,g_rates)!=NUM_RATES)
	{
		Print("CopyRates of ",_Symbol," failed, no history");
		return false;
	}

	//--- get current ATR
	if ( g_handleAtr != INVALID_HANDLE )
	{
		if(CopyBuffer(g_handleAtr,0,0,NUM_RATES,g_atr)!=NUM_RATES)
		{
			Print("CopyBuffer from iATR failed, no data");
			return false;
		}
	}

	return true;
}

// Determines if we have an open position or not
bool SelectPosition(const string symbol, const ulong magicNumber)
{
	// Do we have an open position?
	if(!PositionSelect(symbol))
	{
		// No position is open for this symbol
		return false;
	}
	else
	{
		// We found a position for this symbol, but is it one of ours?
		return(PositionGetInteger(POSITION_MAGIC)==magicNumber);
	}

	return false;
}

// This is called when a trade has completed.
void TradeCompleted(const ulong ticket)
{
	// shorthand values from the history.
	const double profit = HistoryDealGetDouble(ticket,DEAL_PROFIT);
	const double swap = HistoryDealGetDouble(ticket,DEAL_SWAP);
	const double fee = HistoryDealGetDouble(ticket,DEAL_COMMISSION);
	const datetime transactionTime=(datetime)HistoryDealGetInteger(ticket,DEAL_TIME);
	// TP or SL is mentioned in the reason
	const ENUM_DEAL_REASON dealReason=(ENUM_DEAL_REASON) HistoryDealGetInteger(ticket,DEAL_REASON);
	const string dealComment = HistoryDealGetString(ticket,DEAL_COMMENT);

	// The total transction amount includes swap and fee
	const double transactionAmount = ( profit + swap + fee );
/*
 * We can hold intermediate values in global variables if, for example
 * to calculate Z-score.

	// Every transaction (both open & close) pays fee.
	// Include every "open" comission in the losses.
	g_totalLoss+=fee;

	// Now sift the close transaction amount into either the win or loss bucket
	if ( transactionAmount > 0. )
	{
		// Profitable trade
		g_profitTrades++;
		g_totalProfit+=transactionAmount;
	}
	else
	{
		// Losing trade
		g_lossTrades++;
		g_totalLoss+=transactionAmount;
	}
*/
	// Dump transaction to log
	string strDealReason = EnumToString(dealReason);
	StringReplace(strDealReason, "DEAL_REASON_", "");
	Print(StringFormat(	"OUT - Tkt [%d] [%s] Reason [%s] Fee [%.2f] Swap [%.2f] Profit [%.2f] Comment [%s]",
						ticket,
						TimeToString(transactionTime, TIME_DATE|TIME_SECONDS),
						strDealReason,
						fee,
						swap,
						profit,
						dealComment ));
}
